from requests_oauthlib import OAuth1


def build_oauth1(client_key, client_secret, resource_owner_key, resource_owner_secret):
    return OAuth1(
        client_key=client_key, client_secret=client_secret,
        resource_owner_key=resource_owner_key, resource_owner_secret=resource_owner_secret
    )


def clean_dict(dirty_dict):
    """Cleans a dictionary from keys with empty string values"""
    return dict((k, v) for k, v in dirty_dict.iteritems() if v)


class ExceptionWithCode(Exception):
    def __init__(self, code):
        self.code = code


# Deprecated methods
def build_basic_auth(*args, **kwargs):
    raise DeprecationWarning("This method is deprecated since python-requests "
                             "is able to do the same.")



from pyrate.main import Pyrate
from pyrate.utils import clean_dict


class TwitterPyrate(Pyrate):

    # request
    base_url = 'https://api.twitter.com/1.1/'
    default_header_content = None
    default_body_content = None
    auth_data = {
        'type': 'OAUTH1',
        'client_key': None, 'client_secret': None,
        'token_key': None, 'token_secret': None
    }
    send_json = False

    # response
    response_formats = ['json']
    default_response_format = response_formats[0]
    validate_response = True

    connection_check = {
        'http_method': 'GET',
        'target': 'account/verify_credentials'
    }

    def __init__(self, oauth_consumer_key, oauth_consumer_secret, oauth_token,
                 oauth_token_secret, default_response_format=None):
        super(TwitterPyrate, self).__init__()
        self.auth_data['client_key'] = oauth_consumer_key
        self.auth_data['client_secret'] = oauth_consumer_secret
        self.auth_data['token_key'] = oauth_token
        self.auth_data['token_secret'] = oauth_token_secret

        if default_response_format:
            self.default_response_format = default_response_format

    # Convenience
    def tweet(self, status, in_reply_to_status_id=None, loc_lat=None,
              loc_long=None, place_id=None, display_coordinates=None,
              trim_user=None, include_entities=None):

        return self.post('statuses/update', content=clean_dict({
            'status': status, 'in_reply_to_status_id': in_reply_to_status_id,
            'lat': loc_lat, 'long': loc_long, 'place_id': place_id,
            'display_coordinates': display_coordinates, 'trim_user': trim_user,
            'include_entities': include_entities
        }))



#todo: reenable after debugging
#from pyrate.main import Pyrate
import json
from main import Pyrate


class NewrelicPyrate(Pyrate):
    # This variable must be set on instantiation
    api_key = ''

    http_methods = ['GET']
    default_http_method = http_methods[0]
    response_formats = ['json']
    default_body_content = {}
    auth_type = 'API_KEY'
    #connection_check_method = ['POST', 'helper/ping', 'msg', "Everything's Chimpy!"]
    connection_check_method = []
    send_json = True

    def __init__(self, apikey, default_http_method=None, default_return_format=None):
        super(NewrelicPyrate, self).__init__()
        self.api_key = apikey
        self.base_url = 'https://api.newrelic.com/api/v2/'
        self.default_header_content = {
            'X-Api-Key': self.api_key
        }

        if default_http_method:
            self.default_http_method = default_http_method

        if default_return_format or default_return_format == '':
            self.default_response_format = default_return_format



if __name__ == "__main__":
    h = NewrelicPyrate('4f68ecbfe3d6f56f74fa44de3329ec5005c3ad26949136d')

    # check if the connection works
    #h.check_connection()

    # direct api call
    print(h.default_header_content)
    print(h.do('applications', http_method="GET", headers="X-Api-Key :4f68ecbfe3d6f56f74fa44de3329ec5005c3ad26949136d"))


import json
import requests
from utils import build_oauth1

__docformat__ = 'sphinx en'


class Pyrate(object):
    """This is the main class

    :param string base_url: The base url for all api requests
    :param default_header_content: Default content for the HTTP request header
    :param default_body_content: Default content for the HTTP request body
    :param dict auth_type: Dictionary with authentification information. Type
    and personal information like api keys or usernames/passwords
    :param bool send_json: Whether the request body should be encoded with json

    :param list response_formats: List of available return formats for this service
    :param string default_response_format: Default return format (will be used if none else is specified in request)
    :param dict connection_check: Used by :func:`check_connection`
    """

    # request
    base_url = None
    default_header_content = None
    default_body_content = None
    auth_data = {'type': None}
    send_json = False

    # response
    response_formats = []
    default_response_format = None
    validate_response = True

    connection_check = {'http_method': None, 'target': None}

    def __init__(self):
        try:
            self.default_response_format = self.response_formats[0]
        except IndexError:
            self.default_response_format = None

    def request(self, method, target, content=None, request_headers=None,
                response_format=None, return_raw=False):

        request_url = '%s%s' % (self.base_url, target)

        if response_format:
            request_url += '.%s' % response_format
        elif self.default_response_format:
            request_url += '.%s' % self.default_response_format

        request_body = dict(self.default_body_content or {}, **(content or {}))
        request_headers = dict(self.default_header_content or {},
                               **(request_headers or {}))
        response_format = '' + ('.' + self.default_response_format if
                                self.default_response_format else '')
        auth_data = self.get_auth_data()

        # TODO: do we really need this?
        if self.send_json:
            # We need to make sure that body is jsonified
            try:
                request_body = json.dumps(request_body)
            except (TypeError, ValueError):
                pass

        response = requests.request(
            method=method, url=request_url, data=request_body,
            headers=request_headers, auth=auth_data)

        if return_raw:
            return response
        else:
            return self.handle_response(response, response_format)

    def delete(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a DELETE request"""
        return self.request('DELETE', target, content, headers, response_format, return_raw)

    def get(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a GET request"""
        return self.request('GET', target, content, headers, response_format, return_raw)

    def head(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a HEAD request"""
        return self.request('HEAD', target, content, headers, response_format, return_raw)

    def options(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a OPTIONS request"""
        return self.request('OPTIONS', target, content, headers, response_format, return_raw)

    def post(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a POST request"""
        return self.request('POST', target, content, headers, response_format, return_raw)

    def put(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a PUT request"""
        return self.request('PUT', target, content, headers, response_format, return_raw)

    def patch(self, target, content=None, headers=None, response_format=None, return_raw=False):
        """Sends a PATCH request"""
        return self.request('PATCH', target, content, headers, response_format, return_raw)

    def handle_response(self, response, response_format):
        if self.check_response(response):
            try:
                return response.json()
            except (ValueError, TypeError):
                return response.content

    def check_response(self, response):
        if self.validate_response and not 200 <= response.status_code < 300:
            raise Exception(
                "There is something wrong with the response (Code: %i)\n"
                "Request was: %s %s\n"
                "Request data was: %s \n"
                "Response Content: %s" % (
                    response.status_code, response.request.method,
                    response.request.url, response.request.body,
                    response.content))
        return True

    def check_connection(self):
        return self.check_response(self.request(
            method=self.connection_check['http_method'],
            target=self.connection_check['target'],
            return_raw=True
        ))

    def get_auth_data(self):
        try:
            auth_type = self.auth_data['type']
        except IndexError:
            return None

        if not auth_type:
            return None

        elif auth_type == 'BASIC':
            return self.auth_data['username'], self.auth_data['password']

        elif auth_type == 'OAUTH1':
            return build_oauth1(
                client_key=self.auth_data['client_key'],
                client_secret=self.auth_data['client_secret'],
                resource_owner_key=self.auth_data['token_key'],
                resource_owner_secret=self.auth_data['token_secret']
            )

        elif auth_type == 'MANUAL':
            return None

        else:
            raise NotImplementedError()

    # Deprecated Methods
    def do(self, *args, **kwargs):
        raise DeprecationWarning("This proxy method is deprecated. Please use "
                                 "the appropriate method directly instead.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import requests
from requests_oauthlib import OAuth1
from urlparse import parse_qs


def twitter_oauth():
    os.system("clear")
    print("Twitter OAuth")
    print("-------------\n")
    oauth_consumer_key = raw_input("Please enter your OAuth Consumer Key: ")
    oauth_consumer_secret = raw_input("Please enter your"
                                      "OAuth Consumer Secret: ")
    oauth_token, oauth_token_secret = setup_twitter_oauth(
        oauth_consumer_key, oauth_consumer_secret)
    print("These are your OAuth tokens. You need them for the TwitterPyrate.")
    print("OAuth Token: " + oauth_token)
    print("OAuth Token Secret: " + oauth_token_secret)


def setup_twitter_oauth(oauth_consumer_key, oauth_consumer_secret):

    oauth_request_token_url = 'https://api.twitter.com/oauth/request_token'
    oauth_authorize_url = \
        'https://api.twitter.com/oauth/authorize?oauth_token='
    oauth_access_token_url = 'https://api.twitter.com/oauth/access_token'

    # Authorize your app via identifier.
    # Request token
    oauth = OAuth1(oauth_consumer_key, client_secret=oauth_consumer_secret)
    r = requests.post(url=oauth_request_token_url, auth=oauth)
    credentials = parse_qs(r.content)

    resource_owner_key = credentials.get('oauth_token')[0]
    resource_owner_secret = credentials.get('oauth_token_secret')[0]

    # Authorize
    authorize_url = oauth_authorize_url + resource_owner_key
    print('Please go here to authorize:')
    print(authorize_url)

    verifier = raw_input('Please input the verifier: ')
    oauth = OAuth1(oauth_consumer_key,
                   client_secret=oauth_consumer_secret,
                   resource_owner_key=resource_owner_key,
                   resource_owner_secret=resource_owner_secret,
                   verifier=verifier)

    # Finally, Obtain the Access Token
    r = requests.post(url=oauth_access_token_url, auth=oauth)
    credentials = parse_qs(r.content)
    token = credentials.get('oauth_token')[0]
    secret = credentials.get('oauth_token_secret')[0]

    return token, secret


# When were adding more we'll use docopt!
def main():
    os.system("clear")
    print("                         _")
    print("                        | |")
    print("   _ __  _   _ _ __ __ _| |_ ___")
    print("  | '_ \| | | | '__/ _` | __/ _ \ ")
    print("  | |_) | |_| | | | (_| | ||  __/")
    print("  | .__/ \__, |_|  \__,_|\__\___|")
    print("  | |     __/ |")
    print("  |_|    |___/\n\n")

    print("This is Pyrate's commandline tool. For the usage of pyrate,")
    print("please read the docs: https://github.com/Chive/pyrate\n\n")
    print("# Tasks")
    print("1 Generate Twitter OAuth Tokens")
    print("0 Exit\n")
    c = raw_input("Your Choice: ")

    if c == '1':
        twitter_oauth()

    elif c == '0':
        exit("Goodbye")

    else:
        exit("Invalid choice. Goodbye")



from urllib import urlencode

from pyrate.main import Pyrate


class GoogleAnalyticsPyrate(Pyrate):
    # request
    base_url = 'https://www.googleapis.com/analytics/v3/'
    default_header_content = None
    default_body_content = None
    auth_data = {'type': 'MANUAL'}
    send_json = False

    # response
    response_formats = []
    default_response_format = None
    validate_response = True

    connection_check = {'http_method': 'GET', 'target': 'account/info'}

    def __init__(self, token, default_response_format=None):
        super(GoogleAnalyticsPyrate, self).__init__()
        self.auth_data['token'] = token
        self.default_header_content = {
            'Authorization': 'Bearer %s' % token
        }

        if default_response_format or default_response_format == '':
            self.default_response_format = default_response_format

    def get_profiles(self, accountId=None, webPropertyId=None):
        if not accountId:
            accountId = '~all'
        if not webPropertyId:
            webPropertyId = '~all'

        return self.get('management/accounts/%s/webproperties/%s/profiles' % (accountId, webPropertyId))

    def get_data(self, ids, start_date, end_date, metrics, dimensions=None):
        target = 'data/ga?ids=ga:%s&start-date=%s&end-date=%s&metrics=' % (str(ids), start_date, end_date)
        for metric in metrics:
            target += '%s,' % metric
        # remove last trailing comma
        target = target[:-1]

        if dimensions:
            target += '&dimensions='
            for dimension in dimensions:
                target += '%s,' % dimension
            # remove last trailing comma
            target = target[:-1]

        return self.get(target)

'''
    def get_lists(self, modified_since=None):
        # modified_since in ISO-8601 eg: 2014-02-17T08:22:10+00:00
        target = 'lists'
        if modified_since:
            target += '?modified_since=%s' % urlencode(modified_since)

        return self.get(target)

    def get_list_by_id(self, id):
        lists = self.get_lists()
        for list in lists:
            if list['id'] == str(id):
                return list
        return None

    def get_list_by_name(self, name):
        lists = self.get_lists()
        for list in lists:
            if list['name'] == name:
                return list
        return None

    def create_contact(self, email, list_id, action_type):
        # action_type can be either ACTION_BY_OWNER or ACTION_BY_VISITOR
        target = 'contacts?action_by=%s' % action_type
        content = {
            'lists': [{'id': str(list_id)}],
            'email_addresses': [{'email_address': email}]
        }
        r = self.post(
            target=target,
            content=content,
            headers={"Content-Type": "application/json"},
            return_raw=True
        )

        success = False
        if r.status_code == 201:
            success = True
        return success, r.content

'''



import unittest
import sys
from httmock import urlmatch, HTTMock, response

sys.path.append('../pyrate')  # we want the local version and not the installed one
from pyrate.services import basecamp, github, harvest, mailchimp, twitter


# In order to use these tests you need to:
# - copy credentials.py.template -> credentials.py and fill in your credentials
# - copy results.py.template -> results.py and fill in results (get them with a rest-client or requests)

class TestSequenceFunctions(unittest.TestCase):

    def getHandler(self, service):
        if service == 'github':
            return github.GithubPyrate("email@example.com", "mypass")

        elif service == 'harvest':
            return harvest.HarvestPyrate("email@example.com", "mypass", "myorganisation")

        elif service == 'mailchimp':
            return mailchimp.MailchimpPyrate("myapikey-us2")

        elif service == 'twitter':
            return twitter.TwitterPyrate("000", "000", "000", "000")

        elif service == 'basecamp':
            return basecamp.BasecampPyrate("email@example.com", "mypass", "123456")


    # TODO: Improve urlmatch regex's

    ##############################################
    ## TWITTER
    ##############################################

    @urlmatch(netloc=r'api\.twitter\.com')
    def mock_twitter(self, url, request):
        headers = {
            'status_code': 200,
        }
        content = {'id': '1', 'geometry': '0000'}
        return response(200, content, headers)

    def test_twitter_con_do_geo(self):
        h = self.getHandler('twitter')
        with HTTMock(self.mock_twitter):
            self.assertTrue('geometry' in h.get('geo/id/df51dec6f4ee2b2c'))

    def test_twitter_tweet(self):
        h = self.getHandler('twitter')
        with HTTMock(self.mock_twitter):
            self.assertTrue(h.tweet("test"))

    ##############################################
    ## MAILCHIMP
    ##############################################

    @urlmatch(netloc=r'.*\.api\.mailchimp\.com')
    def mock_mailchimp(self, url, request):
        headers = {
            'status_code': 200,
        }
        content = {'msg': "Everything's Chimpy!"}
        return response(200, content, headers)

    def test_mailchimp_con_do(self):
        h = self.getHandler('mailchimp')
        with HTTMock(self.mock_mailchimp):
            self.assertEqual(h.get('helper/ping'), {'msg': "Everything's Chimpy!"})

    def test_mailchimp_con_check(self):
        h = self.getHandler('mailchimp')
        with HTTMock(self.mock_mailchimp):
            self.assertTrue(h.check_connection())

    ##############################################
    ## HARVEST
    ##############################################

    @urlmatch(netloc=r'.*\.harvestapp\.com')
    def mock_harvest(self, url, request):
        headers = {
            'status_code': 200,
            'content-type': 'application/json',
        }
        content = {'user': 'someone', 'company': 'somecompany'}
        return response(200, content, headers)

    def test_harvest_con_do(self):
        h = self.getHandler('harvest')
        with HTTMock(self.mock_harvest):
            res = h.get('account/who_am_i')
        self.assertTrue('company' in res and 'user' in res)

    def test_harvest_con_check(self):
        h = self.getHandler('harvest')
        with HTTMock(self.mock_harvest):
            self.assertTrue(h.check_connection())

    ##############################################
    ## GITHUB
    ##############################################

    @urlmatch(netloc=r'api\.github\.com')
    def mock_github(self, url, request):
        headers = {
            'status_code': 200,
            'content-type': 'application/json',
        }
        content = {'current_user_url': 'github.com/someuser'}
        return response(200, content, headers)

    def test_github_con_do(self):
        h = self.getHandler('github')
        with HTTMock(self.mock_github):
            self.assertTrue('current_user_url' in h.get('#'))

    def test_github_con_check(self):
        h = self.getHandler('github')
        with HTTMock(self.mock_github):
            self.assertTrue(h.check_connection())


    ##############################################
    ## BASECAMP
    ##############################################

    @urlmatch(netloc=r'basecamp\.com')
    def mock_basecamp(self, url, request):
        headers = {
            'status_code': 200,
            'content-type': 'application/json',
        }
        content = {'email_address': "email@example.com"}
        return response(200, content, headers)

    def test_basecamp_con_check(self):
        h = self.getHandler('basecamp')
        with HTTMock(self.mock_basecamp):
            self.assertTrue(h.check_connection())



    # FIXME: Test Suites not working
    '''
    test_basecamp = unittest.TestSuite()
    test_basecamp.addTests(test_basecamp_con_check)

    test_github = unittest.TestSuite()
    test_github.addTests(test_github_con_check)
    test_github.addTests(test_github_con_do)

    test_harvest = unittest.TestSuite()
    test_harvest.addTests(test_harvest_con_check)
    test_harvest.addTests(test_harvest_con_do)

    test_mailchimp = unittest.TestSuite()
    test_mailchimp.addTests(test_mailchimp_con_check)
    test_mailchimp.addTests(test_mailchimp_con_do)

    test_twitter = unittest.TestSuite()
    test_twitter.addTests(test_twitter_con_do_geo)
    test_twitter.addTests(test_twitter_tweet)

    alltests = unittest.TestSuite([test_basecamp, test_github, test_harvest, test_mailchimp, test_twitter])
    '''

if __name__ == '__main__':
    #unittest.TextTestRunner().run(test_basecamp)
    unittest.main()
